from getpass import getpass

import json
from typing import Protocol

from dataclasses import dataclass, field
from dataclasses_json import dataclass_json

from dknovautils import AT
from dknovautils import *


from email.message import EmailMessage

import smtplib, ssl


class EmailUtils:
    """
    https://realpython.com/python-send-email/






    """

    @classmethod
    def rnd_str(cls) -> str:
        return str(AT.fepochMillis())

    @classmethod
    def ssh_email_file(
        cls,
        server: str,
        user: str,
        passwd: str,
        from_id: str,
        to_id: str,
        dirpath: str,
        subject: str,
        message: str,
        port=22,
    ):
        """
        todo

        from_id, to_id:
            a-zA-Z0-9-

        subject 过滤算法



        """

        # pip install scp
        from scp import SCPClient

        import paramiko
        from scp import SCPClient

        # pip install scp

        # cmd = f'sudo mkdir -p "{dirpath}"; '
        # f_ssh(ip=server, user=username, pwd=pwd, port=port, cmd=cmd)

        fname = f"{cls.rnd_str()}.email.txt"
        infofile = DkFile(f"/tmp/{fname}")
        ctt = f"subject:{subject} msg:{message}"
        infofile.path.write_text(data=ctt, encoding="utf-8")

        tfname = f"{from_id}_{to_id}_{AT.sdf_isocompact_format_datetime()}_subject.email.txt"  # todo

        def createSSHClient(server, port, user, password):
            client = paramiko.SSHClient()
            client.load_system_host_keys()
            client.set_missing_host_key_policy(paramiko.AutoAddPolicy())
            client.connect(server, port, user, password)
            return client

        ssh = createSSHClient(server, port, user, passwd)
        scp = SCPClient(ssh.get_transport())
        scp.put(
            infofile.pathstr,
            remote_path=f"{dirpath}/{tfname}",
            preserve_times=True,
        )

        # todo remove infofile

        pass

    @classmethod
    def send_mail_v2024(
        cls,
        mode: str,
        server: str,
        from_email: str,
        to_email,
        subject,
        message,
        password: str,
        username: str | None = None,
        port=465,
    ):
        """

        如果密码错误 会在login的时候出现错误
        比如 Connection unexpectedly closed


        """

        if not username:
            username = from_email

        msg = EmailMessage()
        msg["Subject"] = subject
        msg["From"] = from_email
        msg["To"] = ", ".join(to_email)
        msg.set_content(message)
        print(msg)

        if mode == "a":
            # Create a secure SSL context
            context = ssl.create_default_context()
            with smtplib.SMTP(server, port) as server:
                server.ehlo()
                server.starttls(context=context)
                server.ehlo()
                server.login(username, password)
                # TODO: Send email here
                server.send_message(msg)

        elif mode == "ssl_b":

            smtp = smtplib.SMTP_SSL(server)  # 此处直接一步到位
            smtp.login(username, password)  # 登录SMTP服务
            smtp.send_message(msg)  # 通过SMTP服务器发送邮件
            smtp.quit()

        else:
            AT.never("err94645 bad mode")

        iprint_info("successfully sent the mail.")

        """
        
参考代码
https://www.cnblogs.com/zjdxr-up/p/17872354.html
  
        
20250107 如下代码发送邮件成功

        两个邮箱的授权码都要登录管理一下。长期不用最好重新生成授权码。

        pwd = "xxx"
        EmailUtils.send_mail_v2024(
            mode="b",
            server="smtp.163.com",
            from_email="xxxx@163.com",
            password=pwd,
            to_email=["xxxx@outlook.com", "xxxx@qq.com"],
            subject=f"hello {millis} 大家好 ",
            message=f"Your analysis has done! {millis} 大家好 ",
        )   
        
        pwd = "xxx"
        EmailUtils.send_mail_v2024(
            mode="b",
            server="smtp.qq.com",
            from_email="xxxx@qq.com",
            password=pwd,
            to_email=["xxxx@outlook.com", "xxxx@outlook.com"],
            subject=f"hello {millis} 大家好 ",
            message=f"Your analysis has done! {millis} 大家好 ",
        )             
        
        """

    @classmethod
    def f_send_email(cls):
        AT.unimplemented()
        pass

    """
    

def send_single_email(to_address: str, subject: str, message: str):
    try:
        api_key = os.getenv("MAILGUN_API_KEY")  # get API-Key from the `.env` file

        resp = requests.post(MAILGUN_API_URL, auth=("api", api_key),
                             data={"from": FROM_EMAIL_ADDRESS,
                                   "to": to_address, "subject": subject, "text": message})
        if resp.status_code == 200:  # success
            logging.info(f"Successfully sent an email to '{to_address}' via Mailgun API.")
        else:  # error
            logging.error(f"Could not send the email, reason: {resp.text}")

    except Exception as ex:
        logging.exception(f"Mailgun error: {ex}")

if __name__ == "__main__":
    send_single_email("Manish <manish@exanple.com>", "Single email test", "Testing Mailgun API for a single email")    
    
    

        
    """


"""

这个事情极其无聊，就是smtp经常不能正常工作。
https://www.cnblogs.com/zjdxr-up/p/17872354.html



https://www.emailtooltester.com/en/blog/free-smtp-servers/


https://app.courier.com/
    用gmail帐号登录
    
    似乎也不知道如何简单上手使用。
    
    Your complete platform for sending notifications

    10,000 messages per month for free, no credit card required.
    
    
    curl --request POST \
    --url https://api.courier.com/send \
    --header 'Authorization: Bearer YOUR_AUTH_TOKEN_HERE' \
    --data '{
        "message": {
        "to": {"email":"xxxx@gmail.com"},
        "template": "xxxxxxxx",
        "data": {"recipientName":"hellobaby"}
        }
    }'    




"""



from __future__ import annotations

from typing import Any


def m_add(a: Any, b: Any) -> Any:
    return m_subtract(a + b, 0)


if __name__ == "__main__":
    print("myadd 10")

from .mysubtract import m_subtract


if __name__ == "__main__":
    print("myadd 20")



from __future__ import annotations

import itertools
from typing import List

from dknovautils import commons, iprint_debug, iprint_warn, AT  # type:ignore
from dknovautils.commons import *


_case_list = ["用户名", "密码"]
_value_list = ["正确", "不正确", "特殊符号", "超过最大长度"]


def add_one(number: int) -> int:
    return number + 1


def gen_case(item: List[str] = _case_list, value: List[str] = _value_list) -> None:
    """输出笛卡尔用例集合"""
    for i in itertools.product(item, value):
        print("输入".join(i))


def hello() -> str:
    ver = AT.VERSION
    r = f"welcome allok 大家好 version:{ver}"
    print(r)
    return r


if __name__ == "__main__":
    hello()



from __future__ import annotations

import atomics
from dknovautils.dk_imports import *


_log_msg_CNT = atomics.atomic(width=8, atype=atomics.INT)


def iprint(obj: Any, level: LLevel = LLevel.Debug) -> bool:
    """
    提供True返回值，是为了将该语句直接用于 assert语句中。

    """
    _iprint(obj, level=level)
    return True


def iprint_trace(obj: Any) -> bool:
    return _iprint(obj, level=LLevel.Trace)


def iprint_debug(obj: Any) -> bool:
    return _iprint(obj, level=LLevel.Debug)


def iprint_info(obj: Any) -> bool:
    return _iprint(obj, level=LLevel.Info)


def iprint_warn(obj: Any) -> bool:
    return _iprint(obj, level=LLevel.Warn)


def iprint_error(obj: Any) -> bool:
    return _iprint(obj, level=LLevel.Error)


# 用try语句 只是防止vscode格式化代码 将import语句自动往前移动到文件头部
# 本文件后续会用到AT等对象
try:
    from dknovautils.dkat import *
    from dknovautils.dkipy import *
finally:
    pass


# def _write_async() -> None:
#     AT.unimplemented("err70375 2025删除")
#     pass


# def _dtprint(s: str) -> bool:
#     AT.unimplemented("err75654 2025删除")
#     return iprint(s)


iprint_logging_inited_: bool = False


def _iprint(obj: Any, level: LLevel) -> bool:
    """


    iprint_xxx 函数可以任何时候使用 或者直接使用 没有任何初始化代码就可以直接使用 此时没有文件保存

        如果有执行函数 将执行该函数
        否则 会在没有初始化的时候进行一次内置的初始化 然后执行

    可以调用 init_logging_v1 进行初始化
        将读取yaml配置文件进行初始化
        将设置 _innerLoggerFun_ 用来作为 iprint的执行函数




    """
    # 做一个调用次数的简单计数
    _log_msg_CNT.fetch_inc()

    # 将Trace级别提升一下。不知为何 NotSET 的设置没有生效 并不会打印。
    # 添加了Trace级别
    if False and level == LLevel.Trace:
        level = LLevel.Debug
        pass

    if AT._inner_logger_fun is not None:
        # if level == LLevel.Error:
        #     AT._AstErrorCnt_ += 1
        #     AT._err_msgs.append(str(obj))
        AT._inner_logger_fun(obj, level)
        return True

    if True:
        global iprint_logging_inited_
        if not iprint_logging_inited_:
            # 只在没有配置的时候起作用 不会重复执行
            # 所以如果已经对root logger进行了设置 那么 basicConfig 将 do nothing
            iprint_logging_inited_ = True
            logging.basicConfig(
                level=logging.NOTSET,
                datefmt=AT.STRFMT_ISO_SEC_A,
                format=AT._LOG_FORMAT_106,
            )

            """
            formatter = logging.Formatter('%(asctime)s.%(msecs)03d-%(name)s-%(filename)s-[line:%(lineno)d]'
                                    '-%(levelname)s-[日志信息]: %(message)s',
                                    datefmt='%Y-%m-%d,%H:%M:%S')
            
            """

        AT.logger.log(level.value, obj)
        # logging.log(level.value, obj)

        return True



import dknovautils as dku

from dknovautils.dkat import AT
from dknovautils.myadd import m_add


def test_ccc():

    print(f"version: {AT.VERSION}")

    assert m_add(1, 2) == 3


if __name__ == "__main__":
    test_ccc()
